@@ -1,6 +1,6 @@
 /*
     avicore
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "aviclip.h"
+#include "pixbuf.h"
 #ifdef G_OS_WIN32
 # include <gdk/gdkwin32.h>
 # include <tchar.h>
@@ -31,7 +32,12 @@ enum
 #ifndef G_OS_WIN32
 enum
 {
+# if GTK_CHECK_VERSION(2,6,0)
+  TARGET_VMAID,
+  TARGET_PIXBUF
+# else /* not GTK_CHECK_VERSION(2,6,0) */
   TARGET_VMAID
+# endif /* not GTK_CHECK_VERSION(2,6,0) */
 };
 #endif /* not G_OS_WIN32 */
 
@@ -46,7 +52,6 @@ static gint avi_clip_signals[LAST_SIGNAL] = {0};
 #ifdef G_OS_WIN32
 static UINT uFormat;            /* ja:クリップボード */
 #else /* not G_OS_WIN32 */
-static GdkAtom atom_targets;    /* ja:ターゲット */
 static GdkAtom atom_scenario;   /* ja:シナリオオブジェクト */
 #endif /* not G_OS_WIN32 */
 
@@ -70,7 +75,7 @@ avi_clip_get_type (void)
         NULL,               /* class_finalize */
         NULL,               /* class_data */
         sizeof (AviClip),
-        0,              /* n_preallocs */
+        0,                  /* n_preallocs */
         (GInstanceInitFunc)avi_clip_init,
       };
 
@@ -106,7 +111,6 @@ avi_clip_class_init (AviClipClass *klass)
 #ifdef G_OS_WIN32
   uFormat = RegisterClipboardFormat (_T("Video maid Scenario Object"));
 #else /* not G_OS_WIN32 */
-  atom_targets = gdk_atom_intern ("TARGETS", FALSE);
   atom_scenario = gdk_atom_intern ("Video maid Scenario Object", FALSE);
 #endif /* not G_OS_WIN32 */
 }
@@ -120,10 +124,7 @@ avi_clip_init (AviClip *clip)
   clip->handler_destroy  = 0;
   clip->window           = NULL;
 #ifndef G_OS_WIN32
-  clip->scenario         = NULL;
-  clip->handler_clear    = 0;
-  clip->handler_get      = 0;
-  clip->handler_received = 0;
+  clip->clipboard        = NULL;
 #endif /* not G_OS_WIN32 */
 }
 
@@ -133,99 +134,14 @@ avi_clip_dispose (GObject *object)
 {
   AviClip *clip = AVI_CLIP (object);
 
-#ifndef G_OS_WIN32
-  g_free (clip->scenario);
-  if (clip->handler_clear)
-    g_signal_handler_disconnect (G_OBJECT (clip->window), clip->handler_clear);
-  if (clip->handler_get)
-    g_signal_handler_disconnect (G_OBJECT (clip->window), clip->handler_get);
-  if (clip->handler_received)
-    g_signal_handler_disconnect (G_OBJECT (clip->window),
-                                                    clip->handler_received);
-#endif /* not G_OS_WIN32 */
   if (clip->handler_destroy)
     g_signal_handler_disconnect (G_OBJECT (clip->window),
                                                         clip->handler_destroy);
-
   if (G_OBJECT_CLASS (parent_class)->dispose)
     G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
-#ifndef G_OS_WIN32
-static gboolean
-avi_clip_selection_clear (GtkWidget         *widget,
-                          GdkEventSelection *event,
-                          AviClip           *clip)
-{
-  if (event->selection == GDK_SELECTION_CLIPBOARD)
-    {
-      /* ja:クリップボード */
-      g_free (clip->scenario);
-      clip->scenario = NULL;
-    }
-  return TRUE;
-}
-
-
-static void
-avi_clip_selection_get (GtkWidget        *widget,
-                        GtkSelectionData *data,
-                        guint             info,
-                        guint             time,
-                        AviClip          *clip)
-{
-  /* ja:クリップボード */
-  if (data->selection == GDK_SELECTION_CLIPBOARD
-                                    && clip->scenario && info == TARGET_VMAID)
-    gtk_selection_data_set (data, atom_scenario, 8,
-                        (guchar *)clip->scenario, g_strlen (clip->scenario));
-}
-
-
-static void
-avi_clip_selection_received (GtkWidget        *widget,
-                             GtkSelectionData *data,
-                             guint             time,
-                             AviClip          *clip)
-{
-  if (data->selection == GDK_SELECTION_CLIPBOARD && data->length >= 0)
-    {
-      /* ja:クリップボード */
-      if (data->type == GDK_SELECTION_TYPE_ATOM)
-        {
-          gint i, count;
-          GdkAtom *atoms;
-
-          atoms = (GdkAtom *)data->data;
-          count = data->length / sizeof (GdkAtom);
-          for (i = 0; i < count; i++)
-            if (atoms[i] == atom_scenario)
-              break;
-          if (i < count)
-            gtk_selection_convert (widget, data->selection, atoms[i],
-                                                            GDK_CURRENT_TIME);
-        }
-      else if (data->type == atom_scenario)
-        {
-          AviEdit **avi_edit;
-
-          avi_edit = avi_edit_from_scenario ((gchar *)data->data);
-          if (avi_edit)
-            {
-              gint i;
-
-              g_signal_emit_by_name (G_OBJECT (clip), "paste", avi_edit);
-              for (i = 0; avi_edit[i]; i++)
-                avi_edit_close (avi_edit[i]);
-              g_free (avi_edit);
-            }
-        }
-    }
-}
-#endif /* not G_OS_WIN32 */
-
-
 /******************************************************************************
 *                                                                             *
 * ja:クリップボード関数群                                                     *
@@ -238,11 +154,6 @@ GObject *
 avi_clip_new (GtkWidget *window)
 {
   AviClip *clip;
-#ifndef G_OS_WIN32
-  /* ja:提供可能なセレクション */
-  GtkTargetEntry targets[1]
-                        = {{"Video maid Scenario Object", 0, TARGET_VMAID}};
-#endif /* not G_OS_WIN32 */
 
   if (!window)
     return NULL;
@@ -251,22 +162,77 @@ avi_clip_new (GtkWidget *window)
   clip->handler_destroy = g_signal_connect_swapped (G_OBJECT (window),
                     "destroy", G_CALLBACK (g_object_unref), G_OBJECT (clip));
 #ifndef G_OS_WIN32
-  gtk_selection_add_targets (window, GDK_SELECTION_CLIPBOARD,
-                                            targets, G_N_ELEMENTS (targets));
-  clip->handler_clear = g_signal_connect (G_OBJECT (window),
-                            "selection-clear-event",
-                            G_CALLBACK (avi_clip_selection_clear), clip);
-  clip->handler_get = g_signal_connect (G_OBJECT (window),
-                            "selection-get",
-                            G_CALLBACK (avi_clip_selection_get), clip);
-  clip->handler_received = g_signal_connect (G_OBJECT (window),
-                            "selection-received",
-                            G_CALLBACK (avi_clip_selection_received), clip);
+# if GTK_CHECK_VERSION(2,2,0)
+  clip->clipboard = gtk_widget_get_clipboard (window, GDK_SELECTION_CLIPBOARD);
+# else /* not GTK_CHECK_VERSION(2,2,0) */
+  clip->clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+# endif /* not GTK_CHECK_VERSION(2,2,0) */
 #endif /* not G_OS_WIN32 */
   return G_OBJECT (clip);
 }
 
 
+#ifndef G_OS_WIN32
+# if GTK_CHECK_VERSION(2,6,0)
+struct _AviClipData
+{
+  gchar     *scenario;
+  GdkPixbuf *pixbuf;
+};
+typedef struct _AviClipData AviClipData;
+# endif /* GTK_CHECK_VERSION(2,6,0) */
+
+
+static void
+avi_clip_get_func (GtkClipboard     *clipboard,
+                   GtkSelectionData *selection_data,
+                   guint             info,
+                   gpointer          user_data)
+{
+# if GTK_CHECK_VERSION(2,6,0)
+  if (selection_data->selection == GDK_SELECTION_CLIPBOARD)
+    {
+      AviClipData *clip_data;
+
+      clip_data = user_data;
+      switch (info)
+        {
+          case TARGET_VMAID:
+            gtk_selection_data_set (selection_data, atom_scenario, 8,
+                                            (guchar *)clip_data->scenario,
+                                            g_strlen (clip_data->scenario));
+            break;
+          case TARGET_PIXBUF:
+            gtk_selection_data_set_pixbuf (selection_data, clip_data->pixbuf);
+        }
+    }
+# else /* not GTK_CHECK_VERSION(2,6,0) */
+  if (selection_data->selection == GDK_SELECTION_CLIPBOARD
+                                                    && info == TARGET_VMAID)
+    gtk_selection_data_set (selection_data, atom_scenario, 8,
+                                            user_data, g_strlen (user_data));
+# endif /* not GTK_CHECK_VERSION(2,6,0) */
+}
+
+
+static void
+avi_clip_clear_func (GtkClipboard *clipboard,
+                     gpointer      user_data)
+{
+# if GTK_CHECK_VERSION(2,6,0)
+  AviClipData *clip_data;
+
+  clip_data = user_data;
+  g_free (clip_data->scenario);
+  gdk_pixbuf_unref (clip_data->pixbuf);
+  g_free (clip_data);
+# else /* not GTK_CHECK_VERSION(2,6,0) */
+  g_free (user_data);
+# endif /* not GTK_CHECK_VERSION(2,6,0) */
+}
+#endif /* not G_OS_WIN32 */
+
+
 /*  ja:クリップボードにコピーする
         clip,オブジェクト
     avi_edit,AVI編集ハンドル                                                */
@@ -324,7 +290,7 @@ avi_clip_copy (AviClip  *clip,
                                                         bm_all_bytes (bmih));
                     if (hGlobalBitmap)
                       {
-                        LPBITMAPINFOHEADER lpBmih;;
+                        LPBITMAPINFOHEADER lpBmih;
 
                         lpBmih = GlobalLock (hGlobalBitmap);
                         if (lpBmih)
@@ -368,19 +334,143 @@ avi_clip_copy (AviClip  *clip,
 #else /* not G_OS_WIN32 */
   if (clip)
     {
-      g_free (clip->scenario);
-      clip->scenario = avi_edit_to_scenario (avi_edit, FALSE);
-      if (clip->scenario && !gtk_selection_owner_set (clip->window,
-                                    GDK_SELECTION_CLIPBOARD, GDK_CURRENT_TIME))
+      gchar *scenario;
+# if GTK_CHECK_VERSION(2,6,0)
+      gint i, n_targets;
+      GdkPixbuf *pixbuf = NULL;
+      GtkTargetList *list;
+      GtkTargetEntry *targets;
+# endif /* GTK_CHECK_VERSION(2,6,0) */
+      /* ja:提供可能なセレクション */
+      GtkTargetEntry target[1]
+                        = {{"Video maid Scenario Object", 0, TARGET_VMAID}};
+
+      scenario = avi_edit_to_scenario (avi_edit, FALSE);
+# if GTK_CHECK_VERSION(2,6,0)
+      for (i = 0; avi_edit[i]; i++)
+        if (avi_edit_type (avi_edit[i]) == AVI_TYPE_VIDEO)
+          {
+            AviFrame *avi_frame;
+
+            avi_frame = avi_frame_open (avi_edit[i]);
+            if (avi_frame)
+              {
+                BitmapInfoHeader *bmih;
+
+                bmih = avi_frame_get_bitmap (avi_frame, 0,
+                                        avi_edit_get_width (avi_edit[i]),
+                                        avi_edit_get_height (avi_edit[i]),
+                                        avi_edit_get_bit_count (avi_edit[i]));
+                if (bmih)
+                  pixbuf = pixbuf_from_bitmap (bmih);
+                avi_frame_close (avi_frame);
+                if (pixbuf)
+                  break;
+              }
+          }
+      list = scenario ? gtk_target_list_new (target, G_N_ELEMENTS (target))
+                      : gtk_target_list_new (NULL, 0);
+      if (pixbuf)
+        gtk_target_list_add_image_targets (list, TARGET_PIXBUF, TRUE);
+#  if GTK_CHECK_VERSION(2,10,0)
+      targets = gtk_target_table_new_from_list (list, &n_targets);
+#  else /* not GTK_CHECK_VERSION(2,10,0) */
+      n_targets = g_list_length (list->list);
+      targets = g_malloc (n_targets * sizeof (GtkTargetEntry));
+      for (i = 0; i < n_targets; i++)
         {
-          g_free (clip->scenario);
-          clip->scenario = NULL;
+          GtkTargetPair *pair;
+
+          pair = g_list_nth_data (list->list, i);
+          targets[i].target = gdk_atom_name (pair->target);
+          targets[i].flags  = pair->flags;
+          targets[i].info   = pair->info;
         }
+#  endif /* not GTK_CHECK_VERSION(2,10,0) */
+      gtk_target_list_unref (list);
+      if (targets)
+        {
+          AviClipData *clip_data;
+
+          clip_data = g_malloc (sizeof (AviClipData));
+          clip_data->scenario = scenario;
+          clip_data->pixbuf = pixbuf;
+          gtk_clipboard_clear (clip->clipboard);
+          if (!gtk_clipboard_set_with_data (clip->clipboard,
+                            targets, n_targets,
+                            avi_clip_get_func, avi_clip_clear_func, clip_data))
+            avi_clip_clear_func (clip->clipboard, clip_data);
+#  if GTK_CHECK_VERSION(2,10,0)
+          gtk_target_table_free (targets, n_targets);
+#  else /* not GTK_CHECK_VERSION(2,10,0) */
+          for (i = 0; i < n_targets; i++)
+            g_free (targets[i].target);
+          g_free (targets);
+#  endif /* not GTK_CHECK_VERSION(2,10,0) */
+        }
+# else /* not GTK_CHECK_VERSION(2,6,0) */
+      if (scenario)
+        {
+          gtk_clipboard_clear (clip->clipboard);
+          if (!gtk_clipboard_set_with_data (clip->clipboard,
+                            target, G_N_ELEMENTS (target),
+                            avi_clip_get_func, avi_clip_clear_func, scenario))
+            g_free (scenario);
+        }
+# endif /* not GTK_CHECK_VERSION(2,6,0) */
     }
 #endif /* not G_OS_WIN32 */
 }
 
 
+#ifndef G_OS_WIN32
+static void
+avi_clip_received_func (GtkClipboard     *clipboard,
+                        GtkSelectionData *selection_data,
+                        gpointer          user_data)
+{
+  if (selection_data->selection == GDK_SELECTION_CLIPBOARD
+                                    && selection_data->target == atom_scenario
+                                    && selection_data->type == atom_scenario
+                                    && selection_data->format == 8
+                                    && selection_data->length > 0)
+    {
+      AviEdit **avi_edit;
+
+      avi_edit = avi_edit_from_scenario ((gchar *)selection_data->data);
+      if (avi_edit)
+        {
+          gint i;
+
+          g_signal_emit_by_name (G_OBJECT (user_data), "paste", avi_edit);
+          for (i = 0; avi_edit[i]; i++)
+            avi_edit_close (avi_edit[i]);
+          g_free (avi_edit);
+        }
+    }
+}
+
+
+# if GTK_CHECK_VERSION(2,6,0)
+static void
+avi_clip_image_received_func (GtkClipboard *clipboard,
+                              GdkPixbuf    *pixbuf,
+                              gpointer      user_data)
+{
+  AviEdit *avi_edit[2];
+
+  avi_edit[0] = avi_edit_open_from_pixbuf (pixbuf);
+  if (avi_edit[0])
+    {
+      avi_edit[1] = NULL;
+      g_signal_emit_by_name (G_OBJECT (user_data), "paste", avi_edit);
+      avi_edit_close (avi_edit[0]);
+    }
+}
+# endif /* GTK_CHECK_VERSION(2,6,0) */
+#endif /* not G_OS_WIN32 */
+
+
 /*  ja:クリップボードから貼り付ける
     clip,オブジェクト                                                       */
 void
@@ -473,8 +563,21 @@ avi_clip_paste (AviClip *clip)
         }
     }
 #else /* not G_OS_WIN32 */
+# if GTK_CHECK_VERSION(2,6,0)
   if (clip)
-    gtk_selection_convert (clip->window, GDK_SELECTION_CLIPBOARD, atom_targets,
-                                                            GDK_CURRENT_TIME);
+    {
+      if (gtk_clipboard_wait_is_target_available (clip->clipboard,
+                                                                atom_scenario))
+        gtk_clipboard_request_contents (clip->clipboard, atom_scenario,
+                                                avi_clip_received_func, clip);
+      else if (gtk_clipboard_wait_is_image_available (clip->clipboard))
+        gtk_clipboard_request_image (clip->clipboard,
+                                        avi_clip_image_received_func, clip);
+    }
+# else /* not GTK_CHECK_VERSION(2,6,0) */
+  if (clip)
+    gtk_clipboard_request_contents (clip->clipboard, atom_scenario,
+                                                avi_clip_received_func, clip);
+# endif /* not GTK_CHECK_VERSION(2,6,0) */
 #endif /* not G_OS_WIN32 */
 }
