@@ -807,18 +807,20 @@ avi_file_open_bitmap (const gchar *file)
   if (bmih_get_compression (avi_file->bmih) == BI_COMP_RGB
                 || bmih_get_compression (avi_file->bmih) == BI_COMP_BITFIELDS)
     {
+      gpointer data;
+
       /* ja:無圧縮 */
       size = bm_image_bytes (avi_file->bmih);
-      avi_file->avi_memory = avi_memory_malloc (size);
-      if (fileio_read (avi_file->fio,
-                    avi_memory_get_data (avi_file->avi_memory), size) != size)
+      data = g_malloc (size);
+      if (fileio_read (avi_file->fio, data, size) != size)
         {
-          avi_memory_free (avi_file->avi_memory);
+          g_free (data);
           fileio_close (avi_file->fio);
           g_free (avi_file->bmih);
           g_free (avi_file);
           return NULL;
         }
+      avi_file->avi_memory = avi_memory_register (data, size);
     }
   else if (bmih_get_compression (avi_file->bmih) == BI_COMP_RLE4
                     || bmih_get_compression (avi_file->bmih) == BI_COMP_RLE8)
@@ -847,7 +849,7 @@ avi_file_open_bitmap (const gchar *file)
       size = bm_header_bytes (bmih1);
       avi_file->bmih = g_realloc (avi_file->bmih, size);
       g_memmove (avi_file->bmih, bmih1, size);
-      avi_file->avi_memory = avi_memory_dup ((guint8 *)bmih1 + size,
+      avi_file->avi_memory = avi_memory_alloc ((guint8 *)bmih1 + size,
                                             bm_image_bytes (avi_file->bmih));
       g_free (bmih1);
     }
@@ -1186,7 +1188,7 @@ avi_file_open_memory (gconstpointer  data,
               size = bm_image_bytes (data);
               avi_file->bmih = g_memdup (data, header);
               avi_file->avi_memory
-                            = avi_memory_dup ((guint8 *)data + header, size);
+                            = avi_memory_alloc ((guint8 *)data + header, size);
             }
           else
             {
@@ -1203,7 +1205,7 @@ avi_file_open_memory (gconstpointer  data,
               size = bm_image_bytes (bmih);
               avi_file->bmih = g_memdup (data, header);
               avi_file->avi_memory
-                            = avi_memory_dup ((guint8 *)bmih + header, size);
+                            = avi_memory_alloc ((guint8 *)bmih + header, size);
               g_free (bmih);
             }
           /* ja:インデックス */
@@ -1233,7 +1235,7 @@ avi_file_open_memory (gconstpointer  data,
         {
           /* ja:AVIファイル構造体 */
           avi_file = g_malloc0 (sizeof (AviFile));
-          avi_file->avi_memory = avi_memory_dup ((guint8 *)data
+          avi_file->avi_memory = avi_memory_alloc ((guint8 *)data
                                             + (header + 1) / 2 * 2 + 28, size);
           avi_file->entries = 1;    /* ja:インデックスの数 */
           /* ja:インデックス */
