@@ -1,6 +1,6 @@
 /*
     avicore
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,8 +18,9 @@
 */
 #include "avibase.h"
 #include "avifmt.h"
-#include "gsrfmt.h"
 #include "chunk.h"
+#include "gsrfmt.h"
+#include "pixbuf.h"
 
 
 /******************************************************************************
@@ -33,8 +34,12 @@
 gint
 avi_main_type (const gchar *file)
 {
-  FileIO *fio;
   guint8 buffer[12];            /* ja:ファイル判別用のバッファ */
+  FileIO *fio;
+#if ! GTK_CHECK_VERSION(2,4,0)
+  GdkPixbuf *pixbuf;
+  GError *error = NULL;
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
 
   /* ja:ファイルを開く */
   if (!file || !(fio = fileio_open (file, FILEIO_ACCESS_READ,
@@ -68,6 +73,19 @@ avi_main_type (const gchar *file)
                     && buffer[3] == 'A' && buffer[4] == 'I' && buffer[5] == 'D'
                                 && (buffer[6] == '\n' || buffer[6] == '\r'))
     return AVI_FILE_SCENARIO;/* ja:シナリオオブジェクト */
+#if GTK_CHECK_VERSION(2,4,0)
+  if (gdk_pixbuf_get_file_info (file, NULL, NULL))
+    return AVI_FILE_PIXBUF;
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+  pixbuf = gdk_pixbuf_new_from_file (file, &error);
+  if (error)
+    g_error_free (error);
+  if (pixbuf)
+    {
+      g_object_unref (pixbuf);
+      return AVI_FILE_PIXBUF;
+    }
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   return AVI_FILE_UNKNOWN;
 }
 
@@ -150,6 +168,7 @@ avi_main_streams (const gchar *file)
                                                 ? gmh_get_streams (&gmh) : 0;
         }
       case AVI_FILE_BITMAP:
+      case AVI_FILE_PIXBUF:
       case AVI_FILE_WAVE: return 1;
     }
   return 0;
@@ -391,6 +410,25 @@ avi_edit_open_from_memory (gconstpointer data)
 }
 
 
+/*  ja:AVIをGdkPixbufから開く
+    pixbuf,GdkPixbuf
+       RET,AVI編集ハンドル,NULL:エラー                                      */
+AviEdit *
+avi_edit_open_from_pixbuf (GdkPixbuf *pixbuf)
+{
+  AviEdit *avi_edit = NULL;
+  BitmapInfoHeader *bmih;
+
+  bmih = pixbuf_to_bitmap (pixbuf);
+  if (bmih)
+    {
+      avi_edit = avi_edit_open_from_memory (bmih);
+      g_free (bmih);
+    }
+  return avi_edit;
+}
+
+
 /*  ja:AVIを閉じる
     avi_edit,AVI編集ハンドル
          RET,TRUE:正常終了,FALSE:エラー                                     */
