@@ -145,7 +145,9 @@ signal_destroy_draw (GtkWidget   *widget,
   gint i;
 
   /* ja:メニュー */
-  if (orz_mdi_get_n_pages (ORZ_MDI (mdi)) <= 1)
+  i = orz_mdi_get_n_pages (ORZ_MDI (mdi));
+  if (i < 1
+        || (i == 1 && orz_mdi_get_page_from_data (ORZ_MDI (mdi), vmaid) >= 0))
     {
       set_menu_bar (NULL);
       gtk_statusbar_pop (GTK_STATUSBAR (status),
