@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -80,7 +80,7 @@ signal_activate_menu_history (GtkWidget   *widget,
 
   file_open = g_malloc (sizeof (FileOpen));
   file_open->file = g_strdup (file);
-  file_open->merge = file_merge;
+  file_open->merge = FALSE;
 #ifdef USE_THREAD
   g_thread_create ((GThreadFunc)file_open_edit, file_open, TRUE, NULL);
 #else /* not USE_THREAD */
@@ -187,19 +187,22 @@ signal_drag_data_received (GtkWidget        *widget,
                            guint             info,
                            guint             time)
 {
-  gchar *file, **files;
+  gchar **files;
   gint i;
-  FileOpen *file_open;
 
   files = g_strsplit ((const gchar *)selection_data->data, "\r\n", 0);
   for (i = 0; files[i]; i++)
     {
+      gchar *file;
+
       file = g_filename_from_uri (files[i], NULL, NULL);
       if (file)
         {
+          FileOpen *file_open;
+
           file_open = g_malloc (sizeof (FileOpen));
           file_open->file = file;
-          file_open->merge = file_merge;
+          file_open->merge = FALSE;
 #ifdef USE_THREAD
           g_thread_create ((GThreadFunc)file_open_edit, file_open, TRUE, NULL);
 #else /* not USE_THREAD */
@@ -397,14 +400,14 @@ signal_selection_recv (GObject   *object,
   def_width = array[0];
   def_height = array[1];
   second = arg_x.flag;
-  for (i = 1; argv[i] != NULL; i++)
+  for (i = 1; argv[i]; i++)
     if (arg_files[i])
       {
         FileOpen *file_open;
 
         file_open = g_malloc (sizeof (FileOpen));
         file_open->file = g_strdup (argv[i]);
-        file_open->merge = file_merge;
+        file_open->merge = FALSE;
 #ifdef USE_THREAD
         g_thread_create ((GThreadFunc)file_open_edit, file_open, TRUE, NULL);
 #else /* not USE_THREAD */
