@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ thread_set_menu_bar (gboolean task)
   GtkWidget *menu_shell;
 
   menu_shell = misc_find_menu (menu_entries, "/file/");
-  glist = gtk_container_children (GTK_CONTAINER (menu_shell));
+  glist = gtk_container_get_children (GTK_CONTAINER (menu_shell));
   for (i = g_list_length (glist) - 2; i >= 0; i--)
     gtk_widget_set_sensitive (GTK_WIDGET (g_list_nth_data (glist, i)), task);
   g_list_free (glist);
@@ -235,7 +235,7 @@ gboolean
 thread_delete (GThread *id)
 #else /* not USE_THREAD */
 gboolean
-thread_delete(gint id)
+thread_delete (gint id)
 #endif /* not USE_THREAD */
 {
   gint i;
@@ -262,8 +262,8 @@ thread_delete(gint id)
       if (tl->id == id)
         {
           /* ja:リストにアイテムがあるとき */
-          g_free (tl);
           gtk_list_store_remove (store, &iter);
+          g_free (tl);
         }
     }
 #ifdef USE_THREAD
@@ -281,7 +281,7 @@ thread_delete(gint id)
       switch (tl->id)
         {
           case THREAD_ID_STANDARD: id_standard++; break;
-          case THREAD_ID_CLOSE:    id_close++;    break;
+          case THREAD_ID_CLOSE:    id_close++;
         }
     }
   if (id_standard <= 0)
@@ -434,7 +434,7 @@ close_program (void)
 
   id = g_thread_self ();
 #else /* not USE_THREAD */
-  gint id = 1;
+  gint id = THREAD_ID_CLOSE;
 #endif /* not USE_THREAD */
 
 #ifdef USE_THREAD
