@@ -248,7 +248,7 @@ main (int   argc,
       second = g_key_file_get_boolean (key_file, "pref", "second", NULL);
       def_width = g_key_file_get_integer (key_file, "pref", "width", NULL);
       def_height = g_key_file_get_integer (key_file, "pref", "height", NULL);
-      n_pos = g_key_file_get_integer (key_file, "pref", "n_pos", NULL);
+      n_pos = g_key_file_get_integer (key_file, "pref", "npos", NULL);
       addext = g_key_file_get_boolean (key_file, "vmaid", "addext", NULL);
       default_ratio = g_key_file_get_boolean (key_file, "vmaid", "ratio",
                                                                         NULL);
@@ -632,7 +632,7 @@ VMAID_COPYRIGHT_NOTICE"\n"
       g_key_file_set_boolean (key_file, "pref", "second", second);
       g_key_file_set_integer (key_file, "pref", "width", def_width);
       g_key_file_set_integer (key_file, "pref", "height", def_height);
-      g_key_file_set_integer (key_file, "pref", "n_pos", n_pos);
+      g_key_file_set_integer (key_file, "pref", "npos", n_pos);
       g_key_file_set_boolean (key_file, "vmaid", "addext", addext);
       g_key_file_set_boolean (key_file, "vmaid", "ratio", default_ratio);
       g_key_file_set_boolean (key_file, "vmaid", "merge", file_merge);
